%define anolis_release 2

%define local_arch `arch`
%global build_petitboot 1
# musl does not work
%global build_musl 0
# uclibc does not work.
%global build_uclibc 0
# We really only need this where musl does not exist
%global build_glibc_static 1
# Default
%global default_type glibc
%global print_configs 0

# Some architectures like the hardened flags, others do not.
# If uclibc ever comes back, make variables for it.
%global hcflags %{_hardening_cflags} -fstack-clash-protection
%global hldflags %{_hardening_ldflags} -Wl,-z,relro,-z,now

Name:		busybox
Version:	1.36.0
Release:	%{anolis_release}%{?dist}
Epoch:		1
Summary:	Statically linked binary providing simplified versions of system commands
License:	GPLv2
URL:		http://www.busybox.net
Source0:	http://www.busybox.net/downloads/%{name}-%{version}.tar.bz2
Source2:	busybox-petitboot.config
Source3:	busybox-shared.config
Source4:	busybox-glibc-static.config
Source5:	busybox-uclibc-static.config
Source6:	busybox-musl-static.config
# musl kernel headers
Source10:	https://github.com/sabotage-linux/kernel-headers/archive/refs/tags/v4.19.88-1.tar.gz
Patch0:		busybox-1.31.1-stime-fix.patch
BuildRequires:	gcc
BuildRequires:	libselinux-devel >= 1.27.7-2
BuildRequires:	libsepol-devel
BuildRequires:	libselinux-static
BuildRequires:	libsepol-static
BuildRequires:	glibc-static
%if 0%{?build_musl}
BuildRequires:	musl-libc-static, musl-devel, musl-gcc
%endif
%if 0%{?build_uclibc}
BuildRequires:	uClibc-static
%endif
BuildRequires:	make


# libbb/hash_md5_sha.c
Provides:	bundled(md5-drepper2)

%description
Busybox is a single binary which includes versions of a large number
of system commands, including a shell. This package can be very
useful for recovering from certain types of system failures,
particularly those involving broken shared libraries.

%if 0%{?build_petitboot}
%package petitboot
Summary:	Version of busybox configured for use with petitboot

%description petitboot
Busybox is a single binary which includes versions of a large number
of system commands, including a shell. The version contained in this
package is a minimal configuration intended for use with the Petitboot
bootloader used on PlayStation 3. The busybox package provides a binary
better suited to normal use.
%endif

%package shared
Summary:	A shared (non-static) version of busybox

%description shared
Busybox is a single binary which includes versions of a large number
of system commands, including a shell. The version contained in this
package is build against shared libraries, most notably glibc.

%package doc
Summary:        Documentation files for %{name}
Requires:       %{name} = %{epoch}:%{version}-%{release}
BuildArch:      noarch

%description    doc
The %{name}-doc package contains documentation files for %{name}.

%prep
%setup -q -a 10
%patch0 -p1 -b .stime

%build
# 1. Musl
%if 0%{?build_musl}
# We use musl-libc. It has broader architecture support and is still small.
cp %{SOURCE6} .config
echo "happy orange is arch is : %{local_arch}"
# set all new options to defaults
yes "" | make oldconfig && \
%if 0%{?print_configs}
cat .config && \
%endif
make V=1 \
CC="musl-gcc" \
EXTRA_CFLAGS="-g -Ikernel-headers-4.19.88-1/%{local_arch}/include %{?hcflags}" \
CFLAGS_busybox="-L%{_prefix}/%{local_arch}-linux-musl %{?hldflags}"

cp busybox_unstripped busybox.musl.static
cp docs/busybox.1 docs/busybox.musl.static.1
%endif

make clean

# 2. uclibc
%if 0%{?build_uclibc}
# We use uclibc. It has smaller architecture support, but is more feature rich than musl.
cp %{SOURCE5} .config
# set all new options to defaults
yes "" | make oldconfig && \
%if 0%{?print_configs}
cat .config && \
%endif
make V=1 \
EXTRA_CFLAGS="-fstack-protector-strong -fstack-clash-protection -g -isystem %{_includedir}/uClibc" \
CFLAGS_busybox="-Wl,-z,relro,-z,now -nostartfiles -L%{_libdir}/uClibc %{_libdir}/uClibc/crt*.o"

cp busybox_unstripped busybox.uclibc.static
cp docs/busybox.1 docs/busybox.uclibc.static.1
%endif

make clean

# 3. glibc (static)
%if 0%{?build_glibc_static}
cp %{SOURCE4} .config
# set all new options to defaults
yes "" | make oldconfig && \
%if 0%{?print_configs}
cat .config && \
%endif
make V=1 \
EXTRA_CFLAGS="%{?hcflags} -g" \
CFLAGS_busybox="%{?hldflags}"

cp busybox_unstripped busybox.glibc.static
cp docs/busybox.1 docs/busybox.glibc.static.1
%endif

#    grep -v \
#        -e ^CONFIG_FEATURE_HAVE_RPC \
#        -e ^CONFIG_FEATURE_MOUNT_NFS \
#        -e ^CONFIG_FEATURE_INETD_RPC \
#        .config1 >.config && \
#    echo "# CONFIG_FEATURE_HAVE_RPC is not set" >>.config && \
#    echo "# CONFIG_FEATURE_MOUNT_NFS is not set" >>.config && \
#    echo "# CONFIG_FEATURE_INETD_RPC is not set" >>.config && \

%if 0%{?build_petitboot}

make clean

# 4. Petitboot

# 4a. Musl
%if 0%{?build_musl}
cp %{SOURCE2} .config
# set all new options to defaults
yes "" | make oldconfig
%if 0%{?print_configs}
cat .config && \
%endif
sed -i -e "s/CONFIG_FEATURE_VI_REGEX_SEARCH=y/CONFIG_FEATURE_VI_REGEX_SEARCH=n/" -e "s/CONFIG_EXTRA_COMPAT=y/CONFIG_EXTRA_COMPAT=n/" -e "s/CONFIG_FEATURE_INETD_RPC=y/CONFIG_FEATURE_INETD_RPC=n/" -e "s/CONFIG_FEATURE_UTMP=y/CONFIG_FEATURE_UTMP=n/" .config && \
make V=1 \
CC="musl-gcc" \
EXTRA_CFLAGS="-g -Ikernel-headers-4.19.88-1/%{local_arch}/include %{?hcflags}" \
CFLAGS_busybox="-L%{_prefix}/%{local_arch}-linux-musl %{?hldflags}"

cp busybox_unstripped busybox.musl.petitboot
cp docs/busybox.1 docs/busybox.musl.petitboot.1
%endif

make clean

#4b. Uclibc
%if 0%{?build_uclibc}
cp %{SOURCE2} .config
# set all new options to defaults
yes "" | make oldconfig
%if 0%{?print_configs}
cat .config && \
%endif
sed -i -e "s/CONFIG_UNICODE_PRESERVE_BROKEN=y/CONFIG_UNICODE_PRESERVE_BROKEN=n/" .config && \
make V=1 \
EXTRA_CFLAGS="-g -isystem %{_includedir}/uClibc" \
CFLAGS_busybox="%{_hardening_ldflags} -Wl,-z,relro,-z,now -static -nostartfiles -L%{_libdir}/uClibc %{_libdir}/uClibc/crt1.o %{_libdir}/uClibc/crti.o %{_libdir}/uClibc/crtn.o"; \
LDFLAGS="--static"

cp busybox_unstripped busybox.uclibc.petitboot
cp docs/busybox.1 docs/busybox.uclibc.petitboot.1
%endif

make clean

#4c. Glibc static
%if 0%{?build_glibc_static}
cp %{SOURCE2} .config
# set all new options to defaults
yes "" | make oldconfig
%if 0%{?print_configs}
cat .config && \
%endif
make V=1 \
EXTRA_CFLAGS="-g %{?hcflags}" \
LDFLAGS="%{?hldflags}"

cp busybox_unstripped busybox.glibc.petitboot
cp docs/busybox.1 docs/busybox.glibc.petitboot.1
%endif

%endif

make clean

## Shared
# 5. Glibc

# copy new configuration file
cp %{SOURCE3} .config
# set all new options to defaults
yes "" | make oldconfig
# Use optflags
%if 0%{?print_configs}
cat .config
%endif
make V=1 EXTRA_CFLAGS="%{optflags}" CFLAGS_busybox="%{build_ldflags}"
cp busybox_unstripped busybox.shared
cp docs/busybox.1 docs/busybox.shared.1

%install
mkdir -p %{buildroot}%{_sbindir}
install -m 755 busybox.*.static %{buildroot}%{_sbindir}
mv %{buildroot}%{_sbindir}/busybox.%{default_type}.static %{buildroot}%{_sbindir}/busybox
ln -s ./busybox %{buildroot}%{_sbindir}/busybox.%{default_type}.static
%if 0%{?build_petitboot}
install -m 755 busybox.*.petitboot %{buildroot}%{_sbindir}
mv %{buildroot}%{_sbindir}/busybox.%{default_type}.petitboot %{buildroot}%{_sbindir}/busybox.petitboot
ln -s ./busybox.petitboot %{buildroot}%{_sbindir}/busybox.%{default_type}.petitboot
%endif
install -m 755 busybox.shared %{buildroot}%{_sbindir}/busybox.shared
mkdir -p %{buildroot}%{_mandir}/man1
install -m 644 docs/busybox.*.static.1 %{buildroot}%{_mandir}/man1/
mv %{buildroot}%{_mandir}/man1/busybox.%{default_type}.static.1 %{buildroot}%{_mandir}/man1/busybox.static.1
ln -s ./busybox.static.1 %{buildroot}%{_mandir}/man1/busybox.%{default_type}.static.1
%if 0%{?build_petitboot}
install -m 644 docs/busybox.*.petitboot.1 %{buildroot}%{_mandir}/man1/
mv %{buildroot}%{_mandir}/man1/busybox.%{default_type}.petitboot.1 %{buildroot}%{_mandir}/man1/busybox.petitboot.1
ln -s ./busybox.petitboot.1 %{buildroot}%{_mandir}/man1/busybox.%{default_type}.petitboot.1
%endif
install -m 644 docs/busybox.shared.1 %{buildroot}%{_mandir}/man1/busybox.shared.1

%files
%license LICENSE
%doc README
%{_sbindir}/busybox
%{_sbindir}/busybox*.static
%{_mandir}/man1/busybox*.static.1.*

%if 0%{?build_petitboot}
%files petitboot
%{_sbindir}/busybox*.petitboot
%{_mandir}/man1/busybox*.petitboot.1.*
%endif

%files shared
%{_sbindir}/busybox.shared
%{_mandir}/man1/busybox.shared.1.*

%files doc
%doc README TODO AUTHORS

%changelog
* Mon Mar 27 2023 mgb01105731 <mgb01105731@alibaba-inc.com> - 1:1.36.0-2
- Del Specific Fields

* Wed Jan 4 2023 Funda Wang <fundawang@yeah.net> - 1:1.36.0-1
- New version 1.36.0

* Fri Dec 16 2022 happy_orange <songnannan@linux.alibaba.com> - 1:1.35.0-1
- init package from upstream
